<?php

/**
 * Payment method PagSeguro controller.
 * 
 * @version        $Id$
 * @package        ARTIO Booking
 * @subpackage  	payments
 * @copyright		Copyright (C) 2010 ARTIO s.r.o.. All rights reserved.
 * @author 			ARTIO s.r.o., http://www.artio.net
 * @license     	GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @link        	http://www.artio.net Official website
 */
defined('_JEXEC') or die('Restricted access');

jimport('joomla.log.log');
jimport('joomla.filesystem.file');

JLog::addLogger(array('text_file' => 'plg_bookingpayment_pagseguro.php'), JLog::ALL, array('plg_bookingpayment_pagseguro'));

class BookingPaymentPagSeguro {

    /**
     * Verify PagSeguro response
     * @return boolean
     */
    public function processPayment() {
        try {
            $GLOBALS['PagSeguroTesting'] = $this->pluginParams->get('testing');
            require_once 'PagSeguroLibrary/PagSeguroLibrary.php';

            $notificationType = JRequest::getVar('notificationType');
            $notificationCode = JRequest::getVar('notificationCode');
            $transactionId = JRequest::getVar('transactionId');

            $credentials = new PagSeguroAccountCredentials($this->pluginParams->get('email'), $this->pluginParams->get('token'));

            if ($notificationType == 'TRANSACTION' && $notificationCode) { // auto-notification
                $transaction = PagSeguroNotificationService::checkTransaction($credentials, $notificationCode);
                JLog::add('Notification Code: ' . $notificationCode, JLog::INFO, 'plg_bookingpayment_pagseguro');
            } elseif ($transactionId) { // redirect after payment
                $transaction = PagSeguroTransactionSearchService::searchByCode($credentials, $transactionId);
                JLog::add('Transaction Id: ' . $transactionId, JLog::INFO, 'plg_bookingpayment_pagseguro');
            } else {
                throw new Exception('Invalid response');
            }

            JRequest::setVar('cid', array($transaction->getReference()));

            JLog::add('Reference: ' . $transaction->getReference(), JLog::INFO, 'plg_bookingpayment_pagseguro');
            JLog::add('Status: ' . $transaction->getStatus()->getTypeFromValue(), JLog::INFO, 'plg_bookingpayment_pagseguro');

            switch (JString::strtolower($transaction->getStatus()->getTypeFromValue())) {
                case 'paid':
                    return true; // confirmed
                case 'refunded':
                case 'cancelled':
                    return false; // canceled
                case 'waiting_payment':
                case 'in_analysis':
                case 'available':
                case 'in_dispute':
                case 'initiated':
                    return 0; // pending
                default:
                    throw new Exception('Invalid status ' . $transaction->getStatus()->getValue() . ' ' . $transaction->getStatus()->getTypeFromValue());
            }
        } catch (Exception $e) {
            JLog::add($e->getMessage(), JLog::ERROR, 'plg_bookingpayment_pagseguro');
        }

        return false;
    }

    /**
     * Send request to the PagSeguro
     * @param TableReservation $reservation
     * @param JRegistry $params
     * @return array
     */
    public static function getRequest($reservation, $params, $value, $url) {
        try {

            $GLOBALS['PagSeguroTesting'] = $params->get('testing');
            require_once 'PagSeguroLibrary/PagSeguroLibrary.php';

            $request = new PagSeguroPaymentRequest();

            $request->setCurrency('BRL');
            $request->setReference($reservation->id);
            $request->addParameter('notificationURL', $url);
            $request->addParameter('redirectURL', $url);

            $request->addItem('0001', JText::sprintf('PLG_BOOKINGPAYMENT_PAGSEGURO_PAY_DESC', $reservation->id), 1, ($value = number_format($value, 2)));
            $request->setSender(BookingHelper::formatName($reservation), $reservation->email);

            $credentials = new PagSeguroAccountCredentials($params->get('email'), $params->get('token'));

            JLog::add('Reference: ' . $request->getReference(), JLog::INFO, 'plg_bookingpayment_pagseguro');
            JLog::add('Value: ' . $value, JLog::INFO, 'plg_bookingpayment_pagseguro');
            JLog::add('Sender: ' . $request->getSender()->getName() . ' ' . $request->getSender()->getEmail(), JLog::INFO, 'plg_bookingpayment_pagseguro');
            JLog::add('Response: ' . $url, JLog::INFO, 'plg_bookingpayment_pagseguro');

            $request = $request->register($credentials);

            JLog::add('Request: ' . $request, JLog::INFO, 'plg_bookingpayment_pagseguro');

            return $request;
        } catch (Exception $e) {
            JLog::add($e->getMessage(), JLog::ERROR, 'plg_bookingpayment_pagseguro');
        }
    }

}
