<?php

/**
 * Payment method PayPal.
 * After save reservation display on reservation detail options to pay with PayPal.
 * They are icons with links to PayPal.
 * Primary option to pay full price or option to pay deposit. 
 * 
 * @version		$Id$
 * @package		ARTIO Booking
 * @subpackage  views
 * @copyright	Copyright (C) 2010 ARTIO s.r.o.. All rights reserved.
 * @author 		ARTIO s.r.o., http://www.artio.net
 * @license     GNU/GPL http://www.gnu.org/copyleft/gpl.html
 * @link        http://www.artio.net Official website
 */

defined('_JEXEC') or die('Restricted access');

JHTML::_('behavior.tooltip');

$uri = &JURI::getInstance();
/* @var $uri JURI */
$host = $uri->toString(array('scheme', 'host', 'port'));

/* @var $this BookingViewReservation */
/* @var $payment TablePayment */

// display options to pay according reservation state
switch ($this->reservation->paid) {
	case  RESERVATION_PENDING:
		if ($payment->pay == PAYMENT_PAY_BOTH || $payment->pay == PAYMENT_PAY_FULL) {
			// reservation pending - display option to pay full price - deposit and pay deposit if set
			$price[] = $this->modelReservation->getFullPrice();
			// option's title
			$header[] = JText::_('FULL_RECEIVING');
			// option's tooltip 
			$title[] = $this->escape($payment->title) . '::' . $this->escape(JText::sprintf('PAY_FULL_PRICE_S_BY_S', BookingHelper::displayPrice($this->modelReservation->getFullPrice()), $payment->title));
			// URL to return from PayPal
			$return[] = $host  . JRoute::_(ARoute::payment($payment->alias, $this->reservation->id, 'receive'));
		}
		if ($this->reservation->fullDeposit && ($payment->pay == PAYMENT_PAY_BOTH || $payment->pay == PAYMENT_PAY_DEPOSIT)) { // option to pay deposit
			$price[] = $this->reservation->fullDeposit;
			$header[] = JText::_('DEPOSIT_RECEIVING');
			$title[] = $this->escape($payment->title) . '::' . $this->escape(JText::sprintf('PAY_PRICE_DEPOSIT_S_BY_S', BookingHelper::displayPrice($this->reservation->fullDeposit), $payment->title));
			$return[] = $host . JRoute::_(ARoute::payment($payment->alias, $this->reservation->id, 'receiveDeposit'));		
		}
		break;
	case  RESERVATION_RECEIVE_DEPOSIT:
		if ($payment->pay == PAYMENT_PAY_BOTH || $payment->pay == PAYMENT_PAY_FULL) {
			// only option to pay reservation price
			$price[] = $this->modelReservation->getFullPriceWithoutDeposit();
			$header[] = JText::_('PAY_REST');
			$title[] = $this->escape($payment->title) . '::' . $this->escape(JText::sprintf('PAY_PRICE_REST_S_BY_S', BookingHelper::displayPrice($this->modelReservation->getFullPriceWithoutDeposit()), $payment->title));
			$return[] = $host . JRoute::_(ARoute::payment($payment->alias, $this->reservation->id, 'receive'));
			break;
		}
		default:
	case  RESERVATION_RECEIVE:
		// reservation is receive - nothing to do
		$price = null;
		break;
}
// display clicable icons to redirect to PayPal	
if (is_array($price)) {
	// PayPal in testing mode
	$testing = $params->get('testing', 'No') == 'Yes';
	// Curency used with PayPal
	$currency = $params->get('currency');
	// E-mail of PayPal account
	$email = $params->get('email');
	// Testing or live URL
	$action = $testing ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';
	$testIpn = $testing ? 1 : 0;
	$name = JText::sprintf('RESERVING_S', $this->reservation->subject_title);
	foreach ($price as $key => $value) {
		// display icon for ever pay option (full or deposit)
?>
	<div class="hasTip payment" title="<?php echo $title[$key]; ?>">
		<strong><?php echo $header[$key]; ?></strong>
		<form action="<?php echo $action; ?>" method="post" enctype="application/x-www-form-urlencoded" id="booking_paypal<?php echo $key; ?>">	
			<input class="button" type="image" name="submit" src="http://images.paypal.com/images/x-click-but6.gif" border="0" alt="<?php echo JText::_('PAY'); ?>" />
			<input type="hidden" name="cmd" value="_xclick" />
			<input type="hidden" name="business" value="<?php echo htmlspecialchars($email, ENT_QUOTES); ?>" />
			<input type="hidden" name="receiver_email" value="<?php echo htmlspecialchars($email, ENT_QUOTES); ?>" />
			<input type="hidden" name="item_name" value="<?php echo htmlspecialchars($name, ENT_QUOTES); ?>" />
			<input type="hidden" name="order_id" value="<?php echo htmlspecialchars($this->reservation->id, ENT_QUOTES); ?>" />
			<input type="hidden" name="amount" value="<?php echo htmlspecialchars($price[$key], ENT_QUOTES); ?>" />
			<input type="hidden" name="shipping" value="0.00" />
			<input type="hidden" name="currency_code" value="<?php echo htmlspecialchars($currency, ENT_QUOTES); ?>" />
			<input type="hidden" name="image_url" value="" />
			<input type="hidden" name="return" value="<?php echo str_replace('&amp;', '&', $return[$key]); // return url cannot contain entities  ?>" />
			<input type="hidden" name="notify_url" value="<?php echo str_replace('&amp;', '&', $return[$key]); // return url cannot contain entities  ?>" />
			<input type="hidden" name="cancel_return" value="<?php echo str_replace('&amp;', '&', $return[$key]); // return url cannot contain entities  ?>" />
			<input type="hidden" name="test_ipn" value="<?php echo htmlspecialchars($testIpn, ENT_QUOTES); ?>" />
			<input type="hidden" name="no_shipping" value="1" />
			<input type="hidden" name="no_note" value="1" />
			<input type="hidden" name="bn" value="ARTIOInternationalCo_SP" />
		</form>
	</div>
<?php
	}
	if (count($price) == 1 && $params->get('autosubmit', 'Yes') == 'Yes') {
?>
		<script type="text/javascript">
			// <![CDATA[
			window.addEvent('domready', function() {
				document.id('booking_paypal0').submit();
			});
			// ]]>
		</script>
<?php 
	}
}
?>